package cz.kebrt.unitcheck.api;

import gov.nasa.jpf.JPF;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import cz.kebrt.unitcheck.api.data.TestMethod;
import cz.kebrt.unitcheck.config.Config;
import cz.kebrt.unitcheck.config.ConfigException;
import cz.kebrt.unitcheck.config.TestConfig;
import cz.kebrt.unitcheck.impl.JPFJUnitListener;
import cz.kebrt.unitcheck.jpf.StopExecutionProperty;
import cz.kebrt.unitcheck.junit.SingleTestRunner;
import cz.kebrt.unitcheck.junit.TestMethodExtractor;
import cz.kebrt.unitcheck.junit.VersionDetector;
import cz.kebrt.unitcheck.util.ClassPathBuilder;

/**
 * The main class of UnitCheck. It checks unit tests passed in the configuration.
 *
 * @author michal
 */
public class UnitChecker {

    public static final String JPFJUNIT_LISTENER_KEY = "JPFJUNIT_LISTENER_KEY";
    public static final String UNITCHECK_CONFIG_KEY = "UNITCHECK_CONFIG_KEY";

    private static final Logger log = Logger.getLogger(UnitChecker.class);

    private static final String JPF_NATIVE_CLASSPATH_PREFIX = "+jpf.native_classpath=";
    private static final String VM_CLASSPATH_PREFIX = "+vm.classpath=";
    private static final String VM_SOURCEPATH_PREFIX = "+vm.sourcepath=";

    private static final String[] DEFAULT_VM_CLASSPATH = {
            "unitcheck.jar",
            "junit-4.5.jar",
            "env_jpf.jar",
            "env_jvm.jar" };

    private static final String[] DEFAULT_VM_SOURCEPATH = {
            "unitcheck-src.jar",
            "junit-4.5-src.jar" };

    private static final String[] JPF_CMDLINE_ARGS = {
            "+vm.peer.packages=gov.nasa.jpf.jvm,cz.kebrt.unitcheck.jpf",
            "+vm.store_steps=true",
            "+jpf.report.publisher=" };

    private final Config config;
    private final List<UnitCheckListener> listeners = new ArrayList<UnitCheckListener>();
    private StopExecutionProperty interruptProperty;
    private JPFJUnitListener jpfjunitListener;
    private boolean interrupted = false;

    /**
     * Constructs an instance.
     *
     * @param config configuration of UnitCheck
     */
    public UnitChecker(Config config) {
        this.config = config;
    }

    /**
     * Checks each test passed in the configuration.
     *
     * @throws ConfigException invalid configuration was passed
     */
    public void run() throws ConfigException {
        config.check();
        for (TestConfig test : config.getTests()) {
            if (interrupted) {
                break;
            }
            for (String method : getTestMethods(test)) {
                if (interrupted) {
                    break;
                }
                runTestMethod(test, method);
            }
        }
    }

    /**
     * Interrupts the checking. Activates {@link StopExecutionProperty}.
     */
    public void interrupt() {
        interrupted = true;
        interruptProperty.stop(StopExecutionProperty.USER_ABORT);
    }

    public void kill() {
        interrupted = true;
        jpfjunitListener.finish(interrupted);
    }

    /**
     * Adds a UnitCheck listener that will be notified about checking progress.
     *
     * @param listener UnitCheck listener
     */
    public void addListener(UnitCheckListener listener) {
        listeners.add(listener);
    }

    /**
     * Returns configuration that is used for checking.
     *
     * @return UnitCheck configuration
     */
    public Config getConfig() {
        return config;
    }

    private void runTestMethod(TestConfig test, String method) {
        interruptProperty = new StopExecutionProperty();
        jpfjunitListener = new JPFJUnitListener(this,
                new TestMethod(test.getTestClass().getName(), method),
                listeners, interruptProperty);

        List<String> cmdLine = getCmdLine();
        cmdLine.add(test.getTestClass().getName() + "#" + method);

        for (String s : cmdLine) {
            log.trace(s);
        }

        // create JPF configuration
        gov.nasa.jpf.Config jpfConfig = JPF.createConfig(
                cmdLine.toArray(new String[cmdLine.size()]));

        // add custom objects in JPF configuration
        jpfConfig.put(JPFJUNIT_LISTENER_KEY, jpfjunitListener);
        jpfConfig.put(UNITCHECK_CONFIG_KEY, config);

        // run JPF
        JPF jpf = new JPF(jpfConfig);
        jpf.addListener(jpfjunitListener);
        jpf.addListener(interruptProperty);
        jpf.run();
        jpfjunitListener.finish(interrupted);
    }

    private List<String> getTestMethods(TestConfig test) {
        if (test.getTestMethod() != null) {
            List<String> l = new ArrayList<String>(1);
            l.add(test.getTestMethod());
            return l;
        } else {
            TestMethodExtractor tme = VersionDetector.getTestMethodExtractor(
                    test.getTestClass());
            return tme.getTestMethods();
        }
    }

    private List<String> getCmdLine() {
        List<String> cmdLine = new ArrayList<String>();
        for (String s : JPF_CMDLINE_ARGS) {
            cmdLine.add(s);
        }
        addJPFProperties(cmdLine, config.getJPFProperties());
        if (config.getJPFConfigFile() != null) {
            cmdLine.add("-c=" + config.getJPFConfigFile());
        }

        cmdLine.add(getClassPathString(JPF_NATIVE_CLASSPATH_PREFIX,
                getDefaultVMClasspath(), config.getVMClasspath()));
        cmdLine.add(getClassPathString(VM_CLASSPATH_PREFIX, getDefaultVMClasspath(),
                config.getVMClasspath()));
        cmdLine.add(getClassPathString(VM_SOURCEPATH_PREFIX, getDefaultVMSourcepath(),
                config.getVMSourcepath()));

        log.trace(getClassPathString(VM_CLASSPATH_PREFIX, getDefaultVMClasspath(), config
                .getVMClasspath()));
        log.trace(getClassPathString(VM_SOURCEPATH_PREFIX, getDefaultVMSourcepath(),
                config.getVMSourcepath()));

        cmdLine.add(SingleTestRunner.class.getName());
        return cmdLine;
    }

    private String[] getDefaultVMClasspath() {
        return getLibsPath(DEFAULT_VM_CLASSPATH);
    }

    private String[] getDefaultVMSourcepath() {
        return getLibsPath(DEFAULT_VM_SOURCEPATH);
    }

    private String[] getLibsPath(String[] libs) {
        List<String> ret = new ArrayList<String>();
        for (String s : libs) {
            ret.add(String.format("%s/%s", config.getLibDir(), s));
        }
        return ret.toArray(new String[ret.size()]);
    }

    private void addJPFProperties(List<String> out, Map<String, String> props) {
        for (Map.Entry<String, String> prop : props.entrySet()) {
            out.add(String.format("+%s=%s", prop.getKey(), prop.getValue()));
        }
    }

    private String getClassPathString(String prefix, String[] def, List<String> user) {
        ClassPathBuilder cb = new ClassPathBuilder();
        for (String s : def) {
            cb.add(s);
        }
        cb.add(user);
        return prefix + cb.get(ClassPathBuilder.COMMA_SEPARATOR);
    }

}
