package cz.kebrt.unitcheck.api.data;

import java.util.ArrayList;
import java.util.List;

/**
 * Overall result of checking a test.
 */
public class CheckRunResult {

    private boolean interrupted = false;
    private int checksCount;
    private List<Failure> failures;

    /**
     * Constructs an empty instance.
     */
    public CheckRunResult() {
        failures = new ArrayList<Failure>();
    }

    /**
     * Called when the test is finished. For internal usage only.
     */
    public void checkFinished() {
        ++checksCount;
    }

    /**
     * Adds a failure that occurred while checking the test.
     * For internal usage only.
     *
     * @param failure failure
     */
    public void addFailure(Failure failure) {
        failures.add(failure);
    }

    /**
     * Returns how many times the test finished while it was checked.
     *
     * @return how many times the test finished while it was checked
     */
    public int getChecksCount() {
        return checksCount;
    }

    /**
     * Returns how many failures occurred while the test was checked.
     *
     * @return how many failures occurred while the test was checked
     */
    public int getFailuresCount() {
        return failures.size();
    }

    /**
     * Returns the list of failures that occurred while the test was checked. The list
     * contains zero or exactly one item. More failures may occur when UnitCheck is
     * configured not to stop on the first error (which is not recommended).
     *
     * @return list of failures that occurred while the test was checked
     */
    public List<Failure> getFailures() {
        return failures;
    }

    /**
     * Returns <code>true</code> when the test was successfully checked.
     *
     * @return <code>true</code> when the test was successfully checked
     */
    public boolean isSuccessful() {
        return failures.size() == 0;
    }

    /**
     * Convenient method for returning the the first failure. <code>isSuccessful</code>
     * should be called before to ensure that an error occurred.
     *
     * @return the the first failure
     */
    public Failure getFirstFailure() {
        return failures.get(0);
    }

    /**
     * Was the checking of the test interrupted by a user?
     *
     * @return <code>true</code> when the checking was interrupted
     */
    public boolean isInterrupted() {
        return interrupted;
    }

    /**
     * Sets the interrupted flag. For internal usage only.
     * @param interrupted <code>true</code> when the checking was interrupted
     */
    public void setInterrupted(boolean interrupted) {
        this.interrupted = interrupted;
    }

}
