package cz.kebrt.unitcheck.api.data;

import java.util.ArrayList;
import java.util.List;

/**
 * Describes a transition. Transition is a list of instructions leading from one state to
 * another. Each transition is started by a new choice from a choice generator.
 * Transitions can be traversed in a bidirectional way. Transitions executed by a certain
 * thread can be traversed, too.
 */
public class TransitionHolder {

    private final int ordNumber;
    private final int threadIndex;
    private final ChoiceGeneratorHolder choiceGenerator;
    private final TransitionHolder prevInThread;
    private TransitionHolder nextInThread;
    private final TransitionHolder prev;
    private TransitionHolder next;

    private final List<StepHolder> steps;

    /**
     * Constructs an instance.
     *
     * @param ordNumber ordinal number of the transition
     * @param threadIndex number of the thread executing the transition
     * @param choiceGenerator generator and a particular choice that started the
     *            transition
     * @param prevInThread previous transition executed by the same thread
     * @param prev previously executed transition
     */
    public TransitionHolder(int ordNumber, int threadIndex,
            ChoiceGeneratorHolder choiceGenerator, TransitionHolder prevInThread,
            TransitionHolder prev) {
        this.ordNumber = ordNumber;
        this.threadIndex = threadIndex;
        this.prevInThread = prevInThread;
        this.choiceGenerator = choiceGenerator;
        this.steps = new ArrayList<StepHolder>();
        this.prev = prev;
    }

    /**
     * Adds a step to the transition. For internal usage only.
     *
     * @param step step to be added
     */
    public void addStep(StepHolder step) {
        steps.add(step);
    }

    /**
     * Returns steps that make up the transition.
     *
     * @return steps that make up the transition
     */
    public List<StepHolder> getSteps() {
        return steps;
    }

    /**
     * Returns number of the thread executing the transition.
     *
     * @return number of the thread executing the transition
     */
    public int getThreadIndex() {
        return threadIndex;
    }

    /**
     * Sets the next transition executed by the same thread. For internal usage only.
     * @param transition transition executed by the same thread
     */
    public void setNextInThread(TransitionHolder transition) {
        this.nextInThread = transition;
    }

    /**
     * Returns next transition executed by the same thread.
     *
     * @return next transition executed by the same thread (may be <code>null</code>)
     */
    public TransitionHolder getNextInThread() {
        return nextInThread;
    }

    /**
     * Returns ordinal number of the transition.
     *
     * @return ordinal number of the transition
     */
    public int getOrdNumber() {
        return ordNumber;
    }

    /**
     * Returns previous transition executed by the same thread.
     *
     * @return previous transition executed by the same thread (may be <code>null</code>)
     */
    public TransitionHolder getPrevInThread() {
        return prevInThread;
    }

    /**
     * Returns generator and a particular choice that started the transition.
     *
     * @return generator and a particular choice that started the transition
     */
    public ChoiceGeneratorHolder getChoiceGenerator() {
        return choiceGenerator;
    }

    /**
     * Returns previously executed transition.
     *
     * @return previously executed transition (<code>null</code> for the first transition.
     */
    public TransitionHolder getPrev() {
        return prev;
    }

    /**
     * Sets the next executed transition. For internal usage only.
     * @param next next executed transition
     */
    public void setNext(TransitionHolder next) {
        this.next = next;
    }

    /**
     * Returns next executed transition.
     *
     * @return next executed transition (<code>null</code> for the last transition.
     */
    public TransitionHolder getNext() {
        return next;
    }

}
