package cz.kebrt.unitcheck.jpf;

import gov.nasa.jpf.jvm.MJIEnv;
import gov.nasa.jpf.jvm.Path;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import cz.kebrt.unitcheck.api.UnitChecker;
import cz.kebrt.unitcheck.api.data.Description;
import cz.kebrt.unitcheck.api.data.ExceptionFailure;
import cz.kebrt.unitcheck.config.Config;
import cz.kebrt.unitcheck.junit.JUnitUtil;
import cz.kebrt.unitcheck.junit.TestReportListener;
import cz.kebrt.unitcheck.junit.TestRunListener;
import cz.kebrt.unitcheck.junit.holder.TestRunResult;

/**
 * MJI peer class for {@link TestReportListener}. In each implementation
 * of a native method, it reconstructs information provided by JUnit into
 * custom UnitCheck structures. Afterwards, an instance of {@link TestRunListener}
 * is notified.
 *
 * @author michal
 */
public class JPF_cz_kebrt_unitcheck_junit_TestReportListener {

    public static void testRunStartedNative(MJIEnv env, int robj, int rString0) {
        getListener(env).testRunStarted(constructDescription(env, rString0));
    }

    public static void testRunFinishedNative(MJIEnv env, int robj, boolean v0, int v1,
            int v2, int v3, long v4) {
        getListener(env).testRunFinished(constructResult(env, v0, v1, v2, v3, v4));
    }

    public static void testStartedNative(MJIEnv env, int robj, int rString0) {
        getListener(env).testStarted(constructDescription(env, rString0));
    }

    public static void testFinishedNative(MJIEnv env, int robj, int rString0) {
        getListener(env).testFinished(constructDescription(env, rString0));
    }

    public static void testFailureNative(MJIEnv env, int robj, int rThrowable,
            int rString0, int rString1, int rString2) {
        getListener(env).testFailure(
                constructFailure(env, rThrowable, rString0, rString1, rString2));
    }

    public static void testIgnoredNative(MJIEnv env, int robj, int rString0) {
        getListener(env).testIgnored(constructDescription(env, rString0));
    }

    public static void testAssumptionFailureNative(MJIEnv env, int robj, int rThrowable,
            int rString0, int rString1, int rString2) {
        getListener(env).testFailure(
                constructFailure(env, rThrowable, rString0, rString1, rString2));
    }

    private static TestRunListener getListener(MJIEnv env) {
        return (TestRunListener) env.getConfig().get(UnitChecker.JPFJUNIT_LISTENER_KEY);
    }

    private static Config getUnitCheckConfig(MJIEnv env) {
        return (Config) env.getConfig().get(UnitChecker.UNITCHECK_CONFIG_KEY);
    }

    private static ExceptionFailure constructFailure(MJIEnv env, int rThrowable,
            int rMessage, int rTestHeader, int rTrace) {
        Path path = env.getJPF().getVM().getClonedPath();
        path.add(env.getJPF().getVM().getCurrentTransition());

        return new ExceptionFailure(constructDescription(env, rTestHeader),
                constructThrowable(env, rThrowable), env.getStringObject(rMessage),
                        PathConvertor.convert(path,
                        getUnitCheckConfig(env)), SnapshotConvertor.convert(env.getJPF()
                        .getSearch().getVM()));
    }

    private static Throwable constructThrowable(MJIEnv env, int rThrowable) {
        int msg = env.getReferenceField(rThrowable, "message");
        int cause = env.getReferenceField(rThrowable, "cause");

        Throwable t;
        if (cause > 0) {
            t = new Throwable(env.getStringObject(msg), constructThrowable(env, cause));
        } else {
            t = new Throwable(env.getStringObject(msg));
        }
        t.setStackTrace(constructStackTrace(env, rThrowable));
        return t;
    }

    private static StackTraceElement[] constructStackTrace(MJIEnv env, int rThrowable) {
        int st = env.getReferenceField(rThrowable, "stackTrace");
        int stSize = env.getArrayLength(st);
        Config config = getUnitCheckConfig(env);

        List<StackTraceElement> trace = new ArrayList<StackTraceElement>();
        for (int i = 0; i < stSize; ++i) {
            int el = env.getReferenceArrayElement(st, i);
            int className = env.getReferenceField(el, "className");
            int fileName = env.getReferenceField(el, "fileName");
            int methodName = env.getReferenceField(el, "methodName");

            if (config.isFilterJUnitTrace()
                    && JUnitUtil.isJUnitCode(env.getStringObject(fileName))) {
                continue;
            }

            trace.add(new StackTraceElement(env.getStringObject(className), env
                    .getStringObject(methodName), stripDirectory(env
                    .getStringObject(fileName)), env.getIntField(el, "lineNumber")));
        }
        return trace.toArray(new StackTraceElement[trace.size()]);
    }

    private static Description constructDescription(MJIEnv env, int rDesc) {
        return new Description(env.getStringObject(rDesc));
    }

    private static TestRunResult constructResult(MJIEnv env, boolean successful,
            int failureCount, int ignoreCount, int runCount, long runTime) {
        return new TestRunResult(successful, runCount, ignoreCount,
                failureCount, runTime);
    }

    private static String stripDirectory(String filename) {
        File f = new File(filename);
        return f.getName();
    }

}
