package cz.kebrt.unitcheck.ant;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;
import org.apache.tools.ant.util.ClasspathUtils;

import cz.kebrt.unitcheck.api.UnitChecker;
import cz.kebrt.unitcheck.config.Config;
import cz.kebrt.unitcheck.config.ConfigException;
import cz.kebrt.unitcheck.config.PathPrinterConfig;
import cz.kebrt.unitcheck.config.TestConfig;
import cz.kebrt.unitcheck.console.ConsoleUnitCheckListener;
import cz.kebrt.unitcheck.util.ContextClassPathFactory;

/**
 * UnitCheck Ant task. Allows to run JUnit tests under the JPF model checker.
 * Sample Ant configuration follows.
 *
 * <pre>
 * &lt;unitcheck basedir="${unitcheck.task.dir}"&gt;
 *     &lt;vmclasspath&gt;
 *         &lt;pathelement path="../classes" /&gt;
 *     &lt;/vmclasspath&gt;
 *     &lt;vmsourcepath&gt;
 *          &lt;pathelement path="../src" /&gt;
 *     &lt;/vmsourcepath&gt;
 *     &lt;test class="cz.kebrt.unitcheck.ant.example.HelloWorld" /&gt;
 * &lt;/unitcheck&gt;
 * </pre>
 *
 * @author michal
 */
public class UnitCheckTask extends Task {

    private String baseDir;
    private String configFile;
    private boolean filterJUnitTrace = true;
    private boolean printPath = true;
    private boolean printBytecode = false;
    private boolean traceJRE = false;

    private ClasspathUtils.Delegate vmClasspath;
    private ClasspathUtils.Delegate vmSourcepath;

    private List<BatchTest> batchTests = new ArrayList<BatchTest>();
    private List<SingleTest> singleTests = new ArrayList<SingleTest>();
    private List<JPFProperty> jpfProps = new ArrayList<JPFProperty>();

    @Override
    public void init() {
        vmClasspath = ClasspathUtils.getDelegate(this);
        vmSourcepath = ClasspathUtils.getDelegate(this);
        super.init();
    }

    @Override
    public void execute() throws BuildException {
        ClassLoader backupCl = null;
        try {
            preValidate();

            backupCl = Thread.currentThread().getContextClassLoader();
            Thread.currentThread().setContextClassLoader(
                    this.getClass().getClassLoader());

            Config config = createConfig();
            PathPrinterConfig pathConfig = new PathPrinterConfig(traceJRE, printBytecode);

            UnitChecker uc = new UnitChecker(config);
            uc.addListener(new ConsoleUnitCheckListener(
                    System.out, pathConfig, printPath));

            uc.run();
        } catch (ConfigException e) {
            System.err.println(e.getMessage());
            throw new BuildException(e.getMessage());
        } finally {
            if (backupCl != null) {
                Thread.currentThread().setContextClassLoader(backupCl);
            }
        }
    }

    private void preValidate() {
        File f = new File(baseDir);
        if (!f.isDirectory()) {
            throw new BuildException("basedir not set correctly");
        }
        if (configFile != null) {
            f = new File(configFile);
            if (!f.isFile()) {
                throw new BuildException("conffile not set correctly");
            }
        }
    }

    private Config createConfig() throws ConfigException {
        Config config = new Config();
        config.setFilterJUnitTrace(filterJUnitTrace);
        config.setJPFConfigFile(configFile);
        config.setLibDir(baseDir);

        if (vmClasspath.getClasspath() != null) {
            List<String> cp = Arrays.asList(vmClasspath.getClasspath().list());
            config.setVMClasspath(cp);
            // set classloader to allow loading of input classes
            ClassLoader cl = ContextClassPathFactory.create(cp);
            Thread.currentThread().setContextClassLoader(cl);
        }

        List<String> sourcePath = null;
        if (vmSourcepath.getClasspath() != null) {
            sourcePath = Arrays.asList(vmSourcepath.getClasspath().list());
        } else {
            sourcePath = new ArrayList<String>();
        }
        config.setVMSourcepath(sourcePath);
        config.setJPFProperties(getJPFProperties());

        List<TestConfig> tests = getTests();
        if (tests.size() == 0) {
            throw new BuildException("No test specified");
        }
        config.setTests(tests);

        return config;
    }

    private List<TestConfig> getTests() throws ConfigException {
        List<TestConfig> tests = new ArrayList<TestConfig>();
        for (BatchTest t : batchTests) {
            tests.addAll(t.createTests());
        }
        for (SingleTest t : singleTests) {
            tests.add(t.createTest());
        }
        return tests;
    }

    private Map<String, String> getJPFProperties() {
        Map<String, String> props = new HashMap<String, String>();
        for (JPFProperty prop : jpfProps) {
            props.put(prop.getKey(), prop.getValue());
        }
        return props;
    }

    // =================================================================================
    // The following methods correspond to elements and attributes supported by the task
    // =================================================================================

    /**
     * Creates a batch test and adds it to the collection of input tests.
     *
     * @return batch test
     */
    public BatchTest createBatchTest() {
        BatchTest test = new BatchTest();
        batchTests.add(test);
        return test;
    }

    /**
     * Creates a single test and adds it to the collection of input tests.
     *
     * @return single test
     */
    public SingleTest createTest() {
        SingleTest test = new SingleTest();
        singleTests.add(test);
        return test;
    }

    /**
     * Creates a JPF property and adds to the collection of properties.
     *
     * @return JPF property
     */
    public JPFProperty createJPFProperty() {
        JPFProperty prop = new JPFProperty();
        jpfProps.add(prop);
        return prop;
    }

    /**
     * Sets classpath that will be used by JPF VM for checking test class(es).
     *
     * @param r reference to the classpath
     */
    public void setVMClasspathRef(Reference r) {
        vmClasspath.setClasspathref(r);
    }

    /**
     * Returns classpath that will be used by JPF VM for checking test class(es).
     *
     * @return returns classpath
     */
    public Path createVMClasspath() {
        return vmClasspath.createClasspath();
    }

    /**
     * Sets sourcepath that will be used by JPF VM for checking test class(es).
     *
     * @param r reference to the sourcepath
     */
    public void setVMSourcepathRef(Reference r) {
        vmSourcepath.setClasspathref(r);
    }

    /**
     * Returns sourcepath that will be used by JPF VM for checking test class(es).
     *
     * @return returns sourcepath
     */
    public Path createVMSourcepath() {
        return vmSourcepath.createClasspath();
    }

    /**
     * Sets the directory containing UnitCheck task JAR files.
     *
     * @param dir directory containing UnitCheck task JAR files
     */
    public void setBaseDir(String dir) {
        this.baseDir = dir;
    }

    /**
     * Filter JUnit and UnitCheck code out of error traces.
     *
     * @param filterJUnitTrace filter JUnit and UnitCheck code out of
     *        error traces
     */
    public void setFilterJUnitTrace(boolean filterJUnitTrace) {
        this.filterJUnitTrace = filterJUnitTrace;
    }

    /**
     * Print the execution history (trace) when an error occurs.
     *
     * @param printPath print the execution history (trace) when an error occurs
     */
    public void setPrintPath(boolean printPath) {
        this.printPath = printPath;
    }

    /**
     * Print bytecode instructions in the execution history (trace).
     *
     * @param printBytecode print bytecode instructions in the execution history (trace)
     */
    public void setPrintBytecode(boolean printBytecode) {
        this.printBytecode = printBytecode;
    }

    /**
     * Trace also the standard Java libraries when an error occurs.
     *
     * @param traceJRE trace also the standard Java libraries when an error occurs
     */
    public void setTraceJRE(boolean traceJRE) {
        this.traceJRE = traceJRE;
    }

    /**
     * Path to a JPF configuration file. All default JPF configuration parameters will be
     * overridden with values from this file.
     *
     * @param configFile path to a JPF configuration file
     */
    public void setConfigFile(String configFile) {
        this.configFile = configFile;
    }

}